#include "WeissHapticSensor.h"
#include <Core/util/variants/eigen3/MatrixVariant.h>


using namespace armarx;

WeissHapticSensor::WeissHapticSensor(std::string device)
    : device(device)
{
    sensorTask = new RunningTask<WeissHapticSensor>(this, &WeissHapticSensor::readAndReportSensorValues);
}

void WeissHapticSensor::connect()
{
    //cout << "Open Serial" << endl;
    this->interface.reset(new SerialInterface(device.c_str(), 115200));
    //interface->startLogging(this->deviceFileName + ".transmission.log");
    interface->open();
    //cout << *interface << endl;
    this->sensor.reset(new TactileSensor(interface));

    //cout << "Stop Periodic Frame Acquisition" << endl;
    sensor->stopPeriodicFrameAcquisition();

    //string sensorType = sensor->getSensorType();
    //cout << boost::format("Sensor Type = %1%") % sensorType << endl;

    //tac_system_information_t si = sensor->getSystemInformation();
    //TactileSensor::printSystemInformation(si);

    //sensor->setDeviceTag("Test Tag");
    this->tag = sensor->getDeviceTag();
    //cout << boost::format("Sensor Tag = %1%") % tag << endl;

    //cout << "Tare Sensor Matrix" << endl;
    //sensor->tareSensorMatrix(1);

    //cout << "Get Matrix Information" << endl;
    this->mi = sensor->getMatrixInformation();
    TactileSensor::printMatrixInfo(&mi);

    sensor->setAquisitionWindow(1, 1, mi.res_x, mi.res_y);

    connected = true;
    cout << this << ": Connect done, Interface=" << sensor->getInterfaceInfo() << endl;
}

void WeissHapticSensor::disconnect()
{
    connected = false;
    sensorTask->stop(true);
}

void WeissHapticSensor::setListenerPrx(HapticUnitListenerPrx listenerPrx)
{
    this->listenerPrx = listenerPrx;
}

void WeissHapticSensor::startSampling()
{
    cout << this << ": startSampling" << endl;
    sensorTask->start();
}

void WeissHapticSensor::readAndReportSensorValues()
{
    cout << this << ": readAndReportSensorValues" << endl;
    //bool periodic = false;
    cout << sensor->getInterfaceInfo() << endl;

    cout << this << ": startPeriodicFrameAcquisition" << endl;
    sensor->startPeriodicFrameAcquisition(0);

    while(!sensorTask->isStopped())
    {
        ARMARX_INFO << deactivateSpam(1) << this << ": receicePeriodicFrame";
        PeriodicFrameData data = sensor->receicePeriodicFrame();

        MatrixFloatPtr matrix = new MatrixFloat(mi.res_y, mi.res_x);
        for (int y = 0; y < mi.res_y; y++)
        {
            for (int x = 0; x < mi.res_x; x++)
            {
                short val = data.data[y * mi.res_x + x];
                (*matrix)(y, x) = val;
            }
        }
        listenerPrx->reportSensorValues(tag, matrix);
    }

    cout << this << ": stopPeriodicFrameAcquisition" << endl;
    sensor->stopPeriodicFrameAcquisition();
}
