/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    ArmarXCore::units
 * @author     Peter Kaiser
 * @date       2014
 * @copyright  http://www.gnu.org/licenses/gpl.txt
 *             GNU General Public License
 */

#include "WeissHapticSensorsUnit.h"

#include <Core/robotstate/remote/ArmarPose.h>
#include <Core/robotstate/remote/RobotStateObjectFactories.h>

#include <boost/regex.hpp>
#include <boost/filesystem.hpp>


using namespace armarx;

void WeissHapticSensorsUnit::onInitHapticUnit()
{
    // Read pipe Id property
    /*int firstPipeID = getProperty<int>("FirstPipeID").getValue();
    if(firstPipeID < 0)
    {
        throw UserException("Value of FirstPipeID must be bigger than 0");
    }*/

    // Read pipe base name property
   // std::string pipeBaseName = getProperty<std::string>("PipeBaseName").getValue();

    // Read logfile property
    //std::string logfile = getProperty<std::string>("Logfile").getValue();
    
    //std::vector<std::string> sensors;
    //sensors.push_back("RightSole.HapticMatrix");
    //hapticProtocol.defineSensorCategory(250, sensors);

    // TODO: The task frequency needs to be adapted when the underlying sensor frequency changes
    //sensorTask = new RunningTask<WeissHapticSensorsUnit>(this, &WeissHapticSensorsUnit::readAndReportSensorValues);

    //remoteSystemReady = false;
    /*
    if(!hapticProtocol.openPipes(firstPipeID, firstPipeID + 1, false, true, pipeBaseName))
    {
        throw UserException("Failed to open pipes");
    }

    if(!hapticProtocol.openLogfile(logfile))
    {
        throw UserException("Failed to open logfile");
    }*/

    std::vector<std::string> devices = getDevices();
    /*boost::shared_ptr<WeissHapticSensor> sensor(new WeissHapticSensor(devices.front()));
    this->sensors.push_back(sensor);*/

    for(std::vector<std::string>::iterator it = devices.begin(); it != devices.end(); ++it)
    {
        boost::shared_ptr<WeissHapticSensor> sensor(new WeissHapticSensor(*it));
        this->sensors.push_back(sensor);
    }

    std::cout << "Connect Interfaces" << std::endl;

    //sensors.front()->connect();

    for(std::vector<boost::shared_ptr<WeissHapticSensor> >::iterator it = sensors.begin(); it != sensors.end(); ++it)
    {
        (*it)->connect();
    }

}

std::vector< std::string > WeissHapticSensorsUnit::getDevices()
{
    const std::string target_path( "/dev/" );
    const boost::regex my_filter( "ttyACM[0-9]+" );

    std::vector< std::string > files;

    boost::filesystem::directory_iterator end_itr; // Default ctor yields past-the-end
    for( boost::filesystem::directory_iterator i( target_path ); i != end_itr; ++i )
    {
        // Skip if not a file
        //if( !boost::filesystem::is_( i->status() ) ) continue;

        boost::smatch what;

        // Skip if no match
        if( !boost::regex_match( i->path().filename().string(), what, my_filter ) ) continue;


        // File matches, store it
        files.push_back( "/dev/" + i->path().filename().string() );
    }
    std::sort(files.begin(), files.end());

    std::cout << "Detected ACM-Interfaces: " << std::endl;
    for(std::vector<std::string>::iterator it = files.begin(); it != files.end(); ++it)
    {
        std::cout << *it << std::endl;
    }

    return files;
}

void WeissHapticSensorsUnit::onStartHapticUnit()
{

    for(std::vector<boost::shared_ptr<WeissHapticSensor> >::iterator it = sensors.begin(); it != sensors.end(); ++it)
    {
        (*it)->setListenerPrx(listenerPrx);
        (*it)->startSampling();
    }

}

void WeissHapticSensorsUnit::onExitHapticUnit()
{
    for(std::vector<boost::shared_ptr<WeissHapticSensor> >::iterator it = sensors.begin(); it != sensors.end(); ++it)
    {
        (*it)->disconnect();
    }
}

/*void WeissHapticSensorsUnit::onConnectComponent()
{

}*/

void WeissHapticSensorsUnit::onDisconnectComponent()
{

}


PropertyDefinitionsPtr WeissHapticSensorsUnit::createPropertyDefinitions()
{
    return PropertyDefinitionsPtr(new WeissHapticSensorsUnitPropertyDefinitions(getConfigIdentifier()));
}

